'use strict';

var generateScriptableBanner = require('@scriptables/manifest/generateScriptableBanner');
var findicloud = require('findicloud');
var promises = require('fs/promises');
var os = require('os');
var path = require('path');
var logger = require('./logger');
var utils = require('./utils');

const findICloudBugsUrl = "https://github.com/hexxspark/findicloud/issues";
async function tryReadManifest(filePath, extensions) {
  for (const ext of extensions) {
    try {
      const manifestPath = filePath.replace(/\.[^.]+$/, "") + ext;
      const content = await promises.readFile(manifestPath, "utf-8");
      return JSON.parse(content);
    } catch {
      continue;
    }
  }
  return null;
}
function scriptableDeploy(options = {}) {
  const {
    verbose = false,
    continueOnError = true,
    scriptableDir,
    patterns = ["**/*.{ts,tsx,js,jsx}"],
    addBanner = true,
    manifestExtensions = [".manifest.json", ".manifest", ".json"]
  } = options;
  const filterFile = utils.createPatternMatcher(patterns);
  const logger$1 = logger.createLogger("scriptable-deploy", {
    level: verbose ? "verbose" : "info"
  });
  let targetDir;
  return {
    name: "scriptable-deploy",
    setup(build) {
      if (!build.initialOptions.metafile) {
        logger$1.verbose("Automatically enabling metafile option");
        build.initialOptions.metafile = true;
      }
      build.onStart(async () => {
        try {
          if (scriptableDir) {
            targetDir = scriptableDir;
          } else {
            const [found] = await findicloud.findICloudDrivePaths({ appNamePattern: "Scriptable", types: [findicloud.PathType.APP_STORAGE] });
            if (os.platform() === "darwin" && found?.metadata.contents?.includes("Documents")) {
              targetDir = path.join(found.path, "Documents");
            } else {
              targetDir = found?.path;
            }
          }
          if (!targetDir) {
            throw new Error(
              `Scriptable directory not found in iCloud Drive. Please ensure iCloud Drive is installed and Scriptable app is synced with iCloud. If iCloud Drive is not installed, you can download it from: https://support.apple.com/en-us/HT204283. If the issue persists, please report it at: ${findICloudBugsUrl}`
            );
          }
          logger$1.verbose(`Target directory: ${targetDir}`);
        } catch (error) {
          const message = `Failed to find Scriptable directory: ${error.message}`;
          if (!continueOnError) {
            throw new Error(message);
          }
          logger$1.error(message);
        }
      });
      build.onEnd(async (result) => {
        if (!result.metafile || !targetDir) {
          logger$1.error("Missing required configuration");
          return;
        }
        try {
          const outputs = Object.keys(result.metafile.outputs).filter((file) => {
            const { entryPoint } = result.metafile.outputs[file];
            return entryPoint && filterFile(entryPoint);
          });
          for (const file of outputs) {
            const output = result.metafile.outputs[file];
            const sourceFile = output.entryPoint;
            if (!sourceFile) {
              throw new Error(`No entry point found for ${file}`);
            }
            let content;
            if (result.outputFiles) {
              const outputFile = result.outputFiles.find((f) => utils.isPathEqual(f.path, file));
              if (!outputFile) {
                throw new Error(`Output file not found: ${file}`);
              }
              content = new TextDecoder().decode(outputFile.contents);
            } else {
              await utils.waitForFile(file);
              content = await promises.readFile(file, "utf-8");
            }
            let targetFileName = path.basename(file);
            let manifest = null;
            try {
              manifest = await tryReadManifest(sourceFile, manifestExtensions);
              if (manifest?.name) {
                targetFileName = `${manifest.name}.js`;
              }
            } catch (error) {
              logger$1.warn(`Failed to read manifest for ${file}: ${error}`);
            }
            const targetPath = path.join(targetDir, targetFileName);
            if (addBanner && manifest) {
              try {
                const banner = generateScriptableBanner.generateScriptableBanner(manifest);
                const newContent = banner + "\n" + content;
                await promises.writeFile(targetPath, newContent);
                logger$1.info(`Deployed with banner: ${file} -> ${targetPath}`);
                continue;
              } catch (error) {
                logger$1.warn(`Failed to add banner to ${file}: ${error}`);
              }
            }
            await promises.writeFile(targetPath, content);
            logger$1.info(`Deployed: ${file} -> ${targetPath}`);
          }
          logger$1.info(`Successfully deployed ${outputs.length} files`);
        } catch (error) {
          logger$1.error(`Deploy failed: ${error.message}`);
          if (!continueOnError) {
            throw error;
          }
        }
      });
    }
  };
}

exports.scriptableDeploy = scriptableDeploy;
//# sourceMappingURL=scriptable-deploy.js.map
//# sourceMappingURL=scriptable-deploy.js.map