import { generateScriptableBanner } from "@scriptables/manifest/generateScriptableBanner";
import { findICloudDrivePaths, PathType } from "findicloud";
import { readFile, writeFile } from "fs/promises";
import { platform } from "os";
import { join, basename } from "path";
import { createLogger } from "./logger.js";
import { createPatternMatcher, isPathEqual, waitForFile } from "./utils.js";
const findICloudBugsUrl = "https://github.com/hexxspark/findicloud/issues";
async function tryReadManifest(filePath, extensions) {
    for (const ext of extensions) {
        try {
            const manifestPath = filePath.replace(/\.[^.]+$/, "") + ext;
            const content = await readFile(manifestPath, "utf-8");
            return JSON.parse(content);
        }
        catch {
            continue;
        }
    }
    return null;
}
function scriptableDeploy(options = {}) {
    const { verbose = false, continueOnError = true, scriptableDir, patterns = ["**/*.{ts,tsx,js,jsx}"], addBanner = true, manifestExtensions = [".manifest.json", ".manifest", ".json"] } = options;
    const filterFile = createPatternMatcher(patterns);
    const logger = createLogger("scriptable-deploy", {
        level: verbose ? "verbose" : "info"
    });
    let targetDir;
    return {
        name: "scriptable-deploy",
        setup(build) {
            if (!build.initialOptions.metafile) {
                logger.verbose("Automatically enabling metafile option");
                build.initialOptions.metafile = true;
            }
            build.onStart(async () => {
                try {
                    if (scriptableDir) {
                        targetDir = scriptableDir;
                    }
                    else {
                        const [found] = await findICloudDrivePaths({ appNamePattern: "Scriptable", types: [PathType.APP_STORAGE] });
                        if (platform() === "darwin" && found?.metadata.contents?.includes("Documents")) {
                            targetDir = join(found.path, "Documents");
                        }
                        else {
                            targetDir = found?.path;
                        }
                    }
                    if (!targetDir) {
                        throw new Error(`Scriptable directory not found in iCloud Drive. Please ensure iCloud Drive is installed and Scriptable app is synced with iCloud. If iCloud Drive is not installed, you can download it from: https://support.apple.com/en-us/HT204283. If the issue persists, please report it at: ${findICloudBugsUrl}`);
                    }
                    logger.verbose(`Target directory: ${targetDir}`);
                }
                catch (error) {
                    const message = `Failed to find Scriptable directory: ${error.message}`;
                    if (!continueOnError) {
                        throw new Error(message);
                    }
                    logger.error(message);
                }
            });
            build.onEnd(async (result) => {
                if (!result.metafile || !targetDir) {
                    logger.error("Missing required configuration");
                    return;
                }
                try {
                    const outputs = Object.keys(result.metafile.outputs).filter((file) => {
                        const { entryPoint } = result.metafile.outputs[file];
                        return entryPoint && filterFile(entryPoint);
                    });
                    for (const file of outputs) {
                        const output = result.metafile.outputs[file];
                        const sourceFile = output.entryPoint;
                        if (!sourceFile) {
                            throw new Error(`No entry point found for ${file}`);
                        }
                        let content;
                        if (result.outputFiles) {
                            const outputFile = result.outputFiles.find((f) => isPathEqual(f.path, file));
                            if (!outputFile) {
                                throw new Error(`Output file not found: ${file}`);
                            }
                            content = new TextDecoder().decode(outputFile.contents);
                        }
                        else {
                            await waitForFile(file);
                            content = await readFile(file, "utf-8");
                        }
                        let targetFileName = basename(file);
                        let manifest = null;
                        try {
                            manifest = await tryReadManifest(sourceFile, manifestExtensions);
                            if (manifest?.name) {
                                targetFileName = `${manifest.name}.js`;
                            }
                        }
                        catch (error) {
                            logger.warn(`Failed to read manifest for ${file}: ${error}`);
                        }
                        const targetPath = join(targetDir, targetFileName);
                        if (addBanner && manifest) {
                            try {
                                const banner = generateScriptableBanner(manifest);
                                const newContent = banner + "\n" + content;
                                await writeFile(targetPath, newContent);
                                logger.info(`Deployed with banner: ${file} -> ${targetPath}`);
                                continue;
                            }
                            catch (error) {
                                logger.warn(`Failed to add banner to ${file}: ${error}`);
                            }
                        }
                        await writeFile(targetPath, content);
                        logger.info(`Deployed: ${file} -> ${targetPath}`);
                    }
                    logger.info(`Successfully deployed ${outputs.length} files`);
                }
                catch (error) {
                    logger.error(`Deploy failed: ${error.message}`);
                    if (!continueOnError) {
                        throw error;
                    }
                }
            });
        }
    };
}
export { scriptableDeploy };
//# sourceMappingURL=scriptable-deploy.js.map
//# sourceMappingURL=scriptable-deploy.js.map
